//
//  AppDelegate.swift
//  Lecture15_TimerAppBasicStudents
//
//  Created by Hamza on 6/09/18.
//  Copyright © 2018 Hamza. All rights reserved.
//

import Cocoa

@NSApplicationMain
class AppDelegate: NSObject, NSApplicationDelegate {

    @IBOutlet weak var window: NSWindow!

   // Outlet to the label that displays the formatted timer value
    @IBOutlet weak var outletLabel : NSTextField!
    
    // Outlets to the buttons at the bottom of the window
    @IBOutlet weak var outletReset : NSButton!
    @IBOutlet weak var outletStartStop : NSButton!
       // Outlets to the menus that we added
    @IBOutlet weak var outletStartStopMenu : NSMenuItem!
    @IBOutlet weak var outletResetMenu : NSMenuItem!
    
    // Instance variables to maintain counter state:
    // - the number of seconds that the timer has run for:
    var seconds : Int = 0 ;
      // - true if the timer is currently running
    var running : Bool = false ;
    
    
    // Toggle the state of the timer
    // (startTimer will create NSTimer if needed)
    @IBAction func actionStartStop (_ sender : Any) {
        if ( running == true ){
            stopTimer ();
        } else { // running == false !!
            startTimer ();
        }
    }
     // The reset action zeros the counter
    @IBAction func actionReset (_ sender : Any) {
        resetTimer ();
    }
    
    // Update the label to show the current time, formatted for display
    func updateLabel () {
        // Take the total number of seconds the timer has run for
        var secondsLocal = seconds ;
          // Determine how many hours and minutes this represents
        let hour = secondsLocal/3600;
        secondsLocal %= 3600;
        let min = secondsLocal/60;
        secondsLocal %= 60;
        // Update the label with the formatted timer value
        outletLabel . stringValue = String (
            format :"%02ld:%02ld:%02ld",
            hour ,min , secondsLocal );
    }
    
    
      // Callback method invoked by the timer each second it is running
    @objc func countUp (_ theTimer : Foundation . Timer ){
        if ( running == true ){
              // If our timer is running, update the seconds counter
            self . seconds += 1;
               // ... and update the label
            updateLabel ();
        } else {
             // If our timer is not running, stop this OS timer
            theTimer . invalidate ();
        }
    }
    
// Start the timer
    func startTimer (){
            // Create a timer object that calls the countUp method every second
        let theTimer = Foundation.Timer ( timeInterval :
            1, target : self , selector :
            #selector ( AppDelegate . countUp (_:)) ,
            userInfo : nil , repeats : true );
        // Get a reference to the current event loop
        let loop = RunLoop . current ;
        // Attach the above timer to the event loop:
        // it will then actually start firning
        loop .add( theTimer , forMode :
            RunLoopMode . commonModes );
        // Change our internal state to running
        self . running = true ;
  
        // Update the Start/Stop button label and menu and state of the reset button
        outletStartStop . title = " Stop ";
        outletStartStopMenu . title = " Stop ";
        outletReset . isEnabled = false ;
    }
       // Stop the timer
    func stopTimer () {
        // We are not running the timer anymore
        // ... but the actual NSTimer won't be cleaned up until it next fires.
        // This NSTimer clean-up occurs in the countUp method above.
        self . running = false ;
        
          // Update the Start/Stop button label/Menu and state of the reset button
        outletReset . isEnabled = true ;
        outletStartStop . title = " Start ";
        outletStartStopMenu . title = " Start ";
    }

    func resetTimer (){
        self . seconds = 0;
        updateLabel ();
    }
    
    // When users interact with a menu, the system checks whether items should be greyed out.
    // We use our "running" state to determine whether to grey out start/Stop or Reset.
    override func validateMenuItem(_ menuItem: NSMenuItem) -> Bool {
        if ( menuItem == outletStartStopMenu){
            return true;
        } else if (menuItem == outletResetMenu){
            return !running;
        } else {
            return true;
        }
    }
    
    func applicationDidFinishLaunching(_ aNotification: Notification) {
        // Insert code here to initialize your application
        updateLabel ();
    }

    func applicationWillTerminate(_ aNotification: Notification) {
        // Insert code here to tear down your application
    }


}

